﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using Microsoft.Office.Interop.Excel;

using SuperMap.Desktop;
using SuperMap.Data;
using SuperMap.Mapping;
using SuperMap.UI;
using SuperMap.Desktop.UI;

namespace SuperMap.Desktop.Samples
{
    public partial class Export : UIUserControl
    {
        #region Initialize Panel
        public Export()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Initialize datasource combo box
        /// </summary>
        private void InitCmbDS()
        {
            try
            {
                //Get the current workspace
                Workspace Wkspace = Application.ActiveApplication.Workspace;
                Datasources objDss = Wkspace.Datasources;
                int DsCount = objDss.Count;

                if (DsCount == 0)
                {
                    return;
                }
                else
                {//Add datasource
                    cmbDatasource.Items.Clear();
                    cmbDatasource.BeginUpdate();
                    for (int i = DsCount - 1; i >= 0; i--)
                    {
                        //Define datasource object
                        cmbDatasource.Add(objDss[i]);
                    }
                    cmbDatasource.EndUpdate();
                    cmbDatasource.SelectedIndex = 0;
                }
            }
            catch (System.Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace);
            }
        }

        /// <summary>
        /// Initialize datasource combo box. This function supports adding vector datasets, such as point, line, region, and tabular datasets.
        /// </summary>
        private void InitCmbDt()
        {
            try
            {
                //Get current workspace object
                Workspace Wkspace = Application.ActiveApplication.Workspace;
                Datasources objDss = Wkspace.Datasources;
                int DtvCount = 0;
                //Get current datasource object
                string DsName = cmbDatasource.Text;
                if (DsName == "")
                {
                    return;
                }
                Datasource objDs = objDss[DsName];
                if (objDs != null)
                {
                //Get datasets in the datasource
                Datasets objDts = objDs.Datasets;
                int DtCount = objDts.Count;
                if (DtCount == 0)
                {
                    return;
                }
                else
                {
                    //Add dataset
                    cmbDataset.Items.Clear();
                    cmbDataset.BeginUpdate();
                    for (int i = DtCount - 1; i >= 0; i--)
                    {
                        //Define datasource object
                        Dataset objDt = objDts[i];
                        if (objDt.Type == DatasetType.Point || objDt.Type == DatasetType.Line || objDt.Type == DatasetType.Region ||
                            objDt.Type == DatasetType.Tabular || objDt.Type == DatasetType.CAD || objDt.Type == DatasetType.LineM||
                            objDt.Type == DatasetType.LinkTable || objDt.Type == DatasetType.Network || objDt.Type == DatasetType.Text||
                            objDt.Type == DatasetType.Topology)
                        {
                            string DtName = objDt.Name;
                                cmbDataset.Add(objDt);
                                //cmbDataset.Items.Add(DtName);
                                DtvCount++;
                            }
                            else
                            {
                                continue;
                            }

                        }
                        cmbDataset.EndUpdate();
                        if (DtvCount != 0)
                        {
                            cmbDataset.SelectedIndex = 0;
                        }
                        else
                        {
                        Application.ActiveApplication.Output.Output("Datasource" + DsName + "No point, line, region, or tabular datasets.");
                        }
                    }
                }
                else
                {
                    cmbDataset.Items.Clear();
                }
            }
            catch (System.Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace);
            }
        }

        //operations when dropping down the datasource combo box
        private void cmbDatasource_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                //update values in datasource combox when dropping down it
                InitCmbDt();
            }
            catch (System.Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace);
            }
        }

        //Load method
        private void ExportExcel_Load(object sender, EventArgs e)
        {
            try
            {
            //Datasource combo box
            InitCmbDS();
            //Dataset combo box
            InitCmbDt();
            }
            catch (System.Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace);
            }
        }
        #endregion

        #region Export
        //Specify where to export the attributes
        private void btnLocal_Click(object sender, EventArgs e)
        {
            try
            {
                string strExcelName = string.Empty;
                strExcelName = cmbDataset.Text;

                if (strExcelName == "")
                {
                    Application.ActiveApplication.Output.Output("Please select a datasource that contains vector datasets.");
                    return;
                } 
                else
                {
                    sfdExportExcel.Filter = "Excel Files (*.xls)|*.xls|Excel文件(*.xlsx)|*.xlsx";
                    sfdExportExcel.Title = "Export attributes to Excel";
                    sfdExportExcel.AddExtension = true;
                    sfdExportExcel.FileName = strExcelName;

                    if (sfdExportExcel.ShowDialog() == DialogResult.OK)
                    {
                        string FileName = sfdExportExcel.FileName;
                        txtExportLocaltion.Text = FileName;
                    }
                }
            }
            catch (System.Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace);
            }
        }

        //Export
        private void btnExportExcel_Click(object sender, EventArgs e)
        {
           

            try
            {
                if (File.Exists(txtExportLocaltion.Text.Trim()))
                {
                    DialogResult dialogResult = MessageBox.Show("File already exists, override or not?","Prompt",MessageBoxButtons.YesNo);
                    if (dialogResult==DialogResult.Yes)
                    {
                        File.Delete(txtExportLocaltion.Text.Trim());
                    }
                    else
                    {
                        return;
                    }
                }
                //Get current workspace object
                Workspace workSpace;
                Recordset objRecordset;

                workSpace = Application.ActiveApplication.Workspace;
                //Get current datasource object
                string datasourceName = cmbDatasource.Text;
                Datasource objDatasource = workSpace.Datasources[datasourceName];

                //Get dataset
                string datasetName = cmbDataset.Text;
                if (datasetName == "")
                {
                    return;
                }
                else
                {
                    //Define vector dataset, recordset, and query parameter
                    DatasetVector objDatasetVector;
                    QueryParameter queryParameter= new QueryParameter();

                    //Set vector dataset
                    objDatasetVector = objDatasource.Datasets[datasetName] as DatasetVector;

                    //Set to search through all records
                    queryParameter.AttributeFilter = "1 > 0";
                    queryParameter.HasGeometry = false;
                    queryParameter.CursorType = CursorType.Dynamic;

                    //query result
                    objRecordset = objDatasetVector.Query(queryParameter);
                    if (objRecordset.IsEmpty)
                    {
                        Application.ActiveApplication.Output.Output("Vector dataset " + datasetName + " has no geometry");
                        return;
                    }
                    else
                    {
                        string strFileName = string.Empty;
                        string strTableName = string.Empty;
                        bool bExport = false;
                        int RecCount = objRecordset.RecordCount;

                        //Set
                        strFileName = txtExportLocaltion.Text.Trim();
                        strTableName = objDatasetVector.Name;

                        bExport = ExportToExcel(objRecordset, strFileName, strTableName);

                        if (bExport)
                        {
                            Application.ActiveApplication.Output.Output("Export attributes to " + strFileName + " Successful!");
                        } 
                        else
                        {
                            Application.ActiveApplication.Output.Output("Export failed！");
                        }
                        
                    }
                }
                objRecordset.Close();
                objRecordset.Dispose();
            }
            catch (System.Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace);
            }
        }

        //Export attributes to excel

        public bool ExportToExcel(Recordset objRecordset,string strFileName,string strTableName)
        {
            //The number of records in the recordset


            try
            {
                 int nRSCount = 0;
                 nRSCount = objRecordset.RecordCount;

                 if (nRSCount == 0)
                 {
                     Application.ActiveApplication.Output.Output("Empty dataset. Export failed!");
                     return false;
                 }
                 else
                 {
                     if (strFileName.Length != 0)
                     {
                         Missing miss = Missing.Value;
                         Microsoft.Office.Interop.Excel.ApplicationClass objExcel = new Microsoft.Office.Interop.Excel.ApplicationClass();
                         objExcel.Application.Workbooks.Add(true);

                         //Show excel sheet if excel.Visible is true.
                         objExcel.Visible = false;

                         if (objExcel == null)
                         {
                             Application.ActiveApplication.Output.Output("Unable to start EXCEL！");
                             return false;
                         }
                         Microsoft.Office.Interop.Excel.Workbooks objBooks = (Microsoft.Office.Interop.Excel.Workbooks)objExcel.Workbooks;
                         Microsoft.Office.Interop.Excel.Workbook objBook = (Microsoft.Office.Interop.Excel.Workbook)(objBooks.Add(miss));
                         Microsoft.Office.Interop.Excel.Worksheet objSheet = (Microsoft.Office.Interop.Excel.Worksheet)objBook.ActiveSheet;
                         objSheet.Name = strTableName;

                         //Generate field name
                         int nFieldCount = 0;//The number of fields
                         nFieldCount = objRecordset.FieldCount;
                         FieldInfos objFieldInfos = new FieldInfos();
                         objFieldInfos = objRecordset.GetFieldInfos();
                         if (nFieldCount == 0)
                         {
                             Application.ActiveApplication.Output.Output("The dataset has no attribute fields！");
                             return false;
                         }
                         for (int i = 0; i < nFieldCount; i++)
                         {
                             objExcel.Cells[1, i + 1] = objFieldInfos[i].Name;
                         }

                         //Show progress bar
                         pbExportExcel.Visible = true;
                         pbExportExcel.Maximum = nRSCount;
                         pbExportExcel.Minimum = 0;

                         //Fill data
                         objRecordset.MoveFirst();
                         for (int i = 0; i < nRSCount; i++)
                         {
                             for (int j = 0; j < nFieldCount; j++)
                             {
                                 object Value = objRecordset.GetFieldValue(j);
                                 if (Value == null)
                                 {
                                     objExcel.Cells[i + 2, j + 1] = "";
                                 }
                                 else
                                 {
                                     objExcel.Cells[i + 2, j + 1] = Value.ToString();
                                 }
                             }
                             objRecordset.MoveNext();
                             pbExportExcel.Value++;
                         }

                         pbExportExcel.Value = 0;
                         objSheet.SaveAs(strFileName, miss, miss, miss, miss, miss, Microsoft.Office.Interop.Excel.XlSaveAsAccessMode.xlNoChange, miss, miss, miss);
                         objBook.Close(false, miss, miss);
                         objBooks.Close();
                         objExcel.Quit();
                         Marshal.ReleaseComObject(objSheet);
                         Marshal.ReleaseComObject(objBook);
                         Marshal.ReleaseComObject(objBooks);
                         Marshal.ReleaseComObject(objExcel);
                         GC.Collect();

                         objFieldInfos.Dispose();
                         return true;
                     }
                     else
                     {
                         return false;
                     }
                 }
            }
            catch (System.Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace);
                return false;
            }
        }
        #endregion

    }
}
